#!/bin/sh

PATH=/sbin:/usr/sbin:/bin:/usr/bin
DESC="Init geth"
LAUNCHER_EXE=/usr/lib/go-ethereum-init/geth-launcher.sh
DAEMON=/usr/lib/go-ethereum-init/geth
DAEMON_ARGS=""
STATIC_DEFAULT_CONF=/usr/lib/go-ethereum-init/default/go-ethereum-init
DEFAULT_CONF=/etc/default/go-ethereum-init
LOG_DIR=/var/log
PID_DIR=/var/run/go-ethereum-init
ENABLE="no"

# Exit if the package is not installed
[ -x "$DAEMON" ] || exit 0

if [ -f ${STATIC_DEFAULT_CONF} ]
then
    . ${STATIC_DEFAULT_CONF}
fi

if [ -f ${DEFAULT_CONF} ]
then
    . ${DEFAULT_CONF}
fi

set -e


LOG_FILE=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} ).log
TESTNET_LOG_FILE=${LOG_DIR}/$( /usr/bin/basename ${DAEMON} )-testnet.log

if [ -z "${RUN_VERSION}" ]
then
    RUN_VERSION=both
fi

if ! [ "${ENABLE}" = "yes" ]
then
    /bin/echo "Daemon is disabled check ${DEFAULT_CONF}"
    exit 0
fi

wait_pid_stopped()
{
    PID_WAIT=$1
    IS_RUNNING="1"
    WAIT_COUNTER=0
    WAIT_LIMIT=300

    if [ -n "${PID_WAIT}" ]
    then
	while ! [ "${IS_RUNNING}" = "0" ]
	do
	    IS_RUNNING=`/usr/bin/ps -p ${PID_WAIT} 1>/dev/null 2>/dev/null`
	    IS_RUNNING=$?
	    if [ ${WAIT_COUNTER} -gt ${WAIT_LIMIT} ]
	    then
		IS_RUNNING="0"
	    fi
	    WAIT_COUNTER=$[ ${WAIT_COUNTER} + 1 ]
	    /usr/bin/sleep 1
	done
    fi

    return ${IS_RUNNING}
}

#
# Function that starts the daemon/service
#
do_start()
{
    INIT_TYPE="$1"
    RESULT=0
    if [ "${ENABLE}" = "yes" ]
    then
	if ! [ -f ${PID_DIR} ]
	then
	    /bin/mkdir -p ${PID_DIR}
	fi

	if ! [ -f ${LOG_DIR} ]
	then
	    /bin/mkdir -p ${LOG_DIR}
	fi

	/bin/echo -n "" >> ${LOG_FILE}
	/bin/echo -n "" >> ${TESTNET_LOG_FILE}

	CONFIG_DIR=`/bin/dirname ${SERVER_CONF}`

	if ! [ -d ${CONFIG_DIR} ]
	then
	    /bin/mkdir -p ${CONFIG_DIR}
	fi

	if ! [ -f ${SERVER_CONF} ]
	then
	    /bin/echo -n "" > ${SERVER_CONF}
	fi

	if ! [ -f ${TESTNET_SERVER_CONF} ]
	then
	    /bin/echo -n "" > ${TESTNET_SERVER_CONF}
	fi

	/bin/chown ${RUNUSER}:${RUNGROUP} ${LOG_FILE}
	/bin/chown ${RUNUSER}:${RUNGROUP} ${TESTNET_LOG_FILE}
	/bin/chown -R ${RUNUSER}:${RUNGROUP} ${CONFIG_DIR}
	/bin/chown -R ${RUNUSER}:${RUNGROUP} ${PID_DIR}

	/bin/chmod 0640 ${LOG_FILE}
	/bin/chmod 0640 ${TESTNET_LOG_FILE}
	/bin/chmod 0750 ${CONFIG_DIR}
	/bin/chmod 0640 ${CONFIG_DIR}/*

	/usr/bin/sleep ${INIT_DELAY_DAEMON}

	case "${RUN_VERSION}" in
	    main)
		PIDO=`/bin/su ${RUNUSER} -c "${LAUNCHER_EXE} ${NICE_VAL} ${DAEMON} --config=${SERVER_CONF} ${LOG_FILE}"`
		RESULT=$?
		/bin/echo ${PIDO} > ${PID_DIR}/geth.pid
	    ;;
	    testnet)
		TESTNET_PIDO=`/bin/su ${RUNUSER} -c "${LAUNCHER_EXE} ${NICE_VAL} ${DAEMON} --config=${TESTNET_SERVER_CONF} ${TESTNET_LOG_FILE}"`
		RESULT=$?
		/bin/echo ${TESTNET_PIDO} > ${PID_DIR}/geth-testnet.pid
	    ;;
	    both)
		PIDO=`/bin/su ${RUNUSER} -c "${LAUNCHER_EXE} ${NICE_VAL} ${DAEMON} --config=${SERVER_CONF} ${LOG_FILE}"`
		RESULT=$?
		/bin/sleep 1

		TESTNET_PIDO=`/bin/su ${RUNUSER} -c "${LAUNCHER_EXE} ${NICE_VAL} ${DAEMON} --config=${TESTNET_SERVER_CONF} ${TESTNET_LOG_FILE}"`
		RESULT=$[ ${RESULT} + $? ]
		/bin/echo ${PIDO} > ${PID_DIR}/geth.pid
		/bin/echo ${TESTNET_PIDO} > ${PID_DIR}/geth-testnet.pid
	    ;;
	esac
    else
	echo "Daemon is disabled check ${DEFAULT_CONF}"
	RESULT=0
    fi
    return ${RESULT}
}

#
# Function that stops the daemon/service
#
do_stop()
{
    RESULT=0
    if [ "${ENABLE}" = "yes" ]
    then
	case "${RUN_VERSION}" in
	    main)
		PIDO=`/bin/cat ${PID_DIR}/geth.pid 2>/dev/null`
		/bin/kill -s HUP ${PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
		wait_pid_stopped ${PIDO}
	    ;;
	    testnet)
		TESTNET_PIDO=`/bin/cat ${PID_DIR}/geth-testnet.pid 2>/dev/null`
		/bin/kill -s HUP ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
		wait_pid_stopped ${TESTNET_PIDO}
	    ;;
	    both)
		PIDO=`/bin/cat ${PID_DIR}/geth.pid 2>/dev/null`
		/bin/kill -s HUP ${PIDO} 1>/dev/null 2>/dev/null
		RESULT=$?
		TESTNET_PIDO=`/bin/cat ${PID_DIR}/geth-testnet.pid 2>/dev/null`
		/bin/kill -s HUP ${TESTNET_PIDO} 1>/dev/null 2>/dev/null
		RESULT=$[ ${RESULT} + $? ]
		wait_pid_stopped ${PIDO}
		wait_pid_stopped ${TESTNET_PIDO}
	    ;;
	esac

	COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
	COUNTER=0
	while [ ${COUNT_PROC} -gt 0 ]
	do
	    COUNT_PROC=`/usr/bin/pgrep -f ${DAEMON} | /usr/bin/wc -l`
	    COUNTER=$[ ${COUNTER} + 1 ]
	    if [ ${COUNTER} -ge 95 ]
	    then
		COUNT_PROC=0
	    fi
	    /bin/sleep 1
	done
	/bin/rm -f ${PID_DIR}/geth.pid 1>/dev/null 2>/dev/null
	/bin/rm -f ${PID_DIR}/geth-testnet.pid 1>/dev/null 2>/dev/null
    else
	echo "Daemon is disabled check ${DEFAULT_CONF}"
	RESULT=0
    fi
    return ${RESULT}
}

#
# Function that sends a SIGHUP to the daemon/service
#
do_reload()
{
    do_stop
    /bin/sleep 1
    do_start
}

RESULT=0

case "$1" in
  start)
	/bin/echo "Start geth"
	do_start
	RESULT=$?
	;;
  stop)
	/bin/echo "Stop geth"
	do_stop
	RESULT=$?
	;;
  reload)
	/bin/echo "Reload geth"
	do_reload
	RESULT=$?
	;;
  restart)
	/bin/echo "Restart geth"
	do_reload
	RESULT=$?
	;;
  *)
	/bin/echo "Usage: $SCRIPTNAME {start|stop|reload|restart}" >&2
	exit 3
	;;
esac

exit ${RESULT}
